<?php

namespace WpifyWoo\Admin;

use WC_Settings_Page;
use Wpify\Core_4_0\Traits\ComponentTrait;
use WpifyWoo\Plugin;
use WpifyWoo\WooCommerceIntegration;

/**
 * Class Settings
 * @package WpifyWoo\Admin
 * @property Plugin $plugin
 */
class Settings extends WC_Settings_Page {
	use ComponentTrait;

	public function __construct() {
		$this->id    = WooCommerceIntegration::OPTION_NAME;
		$this->label = __( 'Wpify Woo', 'wpify-woo' );

		/**
		 *  Define all hooks instead of inheriting from parent
		 */

		parent::__construct();

		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

		// Add the tab to the tabs array
		add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_settings_page' ), 99 );

		// Add new section to the page
		add_action( 'woocommerce_sections_' . $this->id, array( $this, 'output_sections' ) );

		// Add settings
		add_action( 'woocommerce_settings_' . $this->id, array( $this, 'output' ) );

		add_filter( 'wpify_woo_settings_general', array( $this, 'settings_general' ) );
	}

	public function enqueue_admin_scripts() {
		$rest_url = $this->get_plugin()->get_api_manager()->get_rest_url();
		$handle   = $this->get_plugin()->get_webpack_manifest()->register_asset(
				'settings.js',
				'wpify-settings',
				array(
						'wpifyWooSettings' => array(
								'publicPath'    => $this->plugin->get_asset_url( 'build/' ),
								'restUrl'       => $this->plugin->get_api_manager()->get_rest_url(),
								'nonce'         => wp_create_nonce( $this->get_plugin()->get_api_manager()->get_nonce_action() ),
								'activateUrl'   => $rest_url . '/license/activate',
								'deactivateUrl' => $rest_url . '/license/deactivate',
								'apiKey'        => $this->plugin->get_license()::API_KEY,
								'apiSecret'     => $this->plugin->get_license()::API_SECRET,
						),
				)
		);

		wp_enqueue_script( $handle );
		wp_set_script_translations( $handle, 'wpify-woo', $this->get_plugin()->get_asset_path( 'languages' ) );
		wp_enqueue_style( $this->get_plugin()->get_webpack_manifest()->register_asset( 'settings.css' ) );
	}

	/**
	 *  Output sections
	 */
	public function output_sections() {
		global $current_section;

		$sections = $this->get_sections();

		if ( empty( $sections ) || 1 === sizeof( $sections ) ) {
			return;
		}

		$array_keys = array_keys( $sections );

		echo '<ul class="subsubsub">';

		foreach ( $sections as $id => $label ) {
			echo '<li><a href="' . admin_url( 'admin.php?page=wc-settings&tab=' . $this->id . '&section=' . sanitize_title( $id ) ) . '" class="' . ( $current_section == $id ? 'current' : '' ) . '">' . $label . '</a> ' . ( end( $array_keys ) == $id ? '' : '|' ) . ' </li>';
		}

		echo '</ul><br class="clear" />';
	}

	/**
	 *  Get sections
	 * @return array
	 */
	public function get_sections(): array {
		$sections = array(
				'' => __( 'General', 'wpify-woo' ),
		);

		return apply_filters( 'woocommerce_get_sections_' . $this->id, $sections );
	}

	/**
	 *  Output the settings
	 */
	public function output() {
		global $current_section;

		if ( ! $current_section ) {
			$current_section = 'general';
		}

		$settings = $this->get_settings();
		$option   = sprintf( '%s-%s', $this->id, $current_section );
		$options  = array();
		$module   = $this->plugin->get_modules_manager()->get_module_by_id( $current_section );
		$slug     = '';


		if ( $module ) {
			$values = $module->get_settings();
		} else {
			$values = $this->get_plugin()->get_woocommerce_integration()->get_settings( $current_section );
		}

		foreach ( $settings as $setting ) {
			if ( 'options' === $setting['type'] ) {
				$options = $setting['options'];
			}
			if ( 'license' === $setting['type'] ) {
				$slug = $setting['slug'];
			}
		}
		?>
		<style type="text/css">
			.wpify-woo-settings__wrapper {
				margin-top: 20px;
				display: flex;
			}

			.wpify-woo-settings {
				width: 70%;
			}

			.wpify-woo-settings__upsells-wrapper {
				background: white;
				padding: 20px;
				width: 30%;
			}

			.wpify-woo-settings__upsells {
				display: flex;
				flex-wrap: wrap;
				justify-content: space-between;
			}

			.wpify-woo-settings__upsell {
				width: 45%;
				margin-bottom: 10px;
				padding: 10px;
				-webkit-box-shadow: 0px 0px 5px 0px rgba(181, 181, 181, 1);
				-moz-box-shadow: 0px 0px 5px 0px rgba(181, 181, 181, 1);
				box-shadow: 0px 0px 5px 0px rgba(181, 181, 181, 1);
			}

			.wpify-woo-settings__upsell h3 {
				margin: 0;
			}

			@media all and (max-width: 767px) {
				.wpify-woo-settings__wrapper, .wpify-woo-settings__upsells {
					display: block;
				}

				.wpify-woo-settings__upsell, .wpify-woo-settings__upsells-wrapper {
					width: 100%;
				}
			}
		</style>
		<div class="wpify-woo-settings__wrapper">
			<div
					class="wpify-woo-settings"
					data-app='wpify-woo-settings'
					id="<?php printf( 'wpify-woo-settings-%s', $current_section ); ?>"
					data-settings="<?php echo esc_attr( json_encode( $settings ) ); ?>"
					data-option="<?php echo esc_attr( $option ); ?>"
					data-current-section="<?php echo esc_attr( $current_section ); ?>"
					data-values="<?php echo esc_attr( json_encode( $values ) ); ?>"
					data-options="<?php echo esc_attr( json_encode( $options ) ); ?>"
					<?php if ( $slug ): ?>
						data-slug="<?php echo esc_attr( $slug ); ?>"
						data-activated="<?php echo $module->is_activated() ? 1 : 0; ?>"
						data-moduleId="<?php echo $module->id(); ?>"
					<?php endif; ?>
			></div>
			<div class="wpify-woo-settings__upsells-wrapper">
				<h3>
					<?php _e( 'Do you enjoy this plugin?', 'wpify-woo' ) ?>
					<a href="https://wordpress.org/support/plugin/wpify-woo/reviews/">
						<?php _e( 'Rate us on Wordpress.org', 'wpify-woo' ) ?>
					</a>
				</h3>
				<h2><?php _e( 'Get premium extensions!', 'wpify-woo' ) ?></h2>
				<div class="wpify-woo-settings__upsells">
					<?php foreach ( $this->plugin->get_premium()->get_extensions() as $extension ) { ?>
						<div class="wpify-woo-settings__upsell">
							<h3><?php echo $extension['title']; ?></h3>
							<ul>
								<?php foreach ( $extension['html_description'] as $item ) { ?>
									<li><?php echo $item; ?></li>
								<?php } ?>
							</ul>
							<a href="<?php echo esc_url( $extension['url'] ); ?>"
							   target="_blank"><?php _e( 'Get now!', 'wpify-woo' ); ?></a>
						</div>
					<?php } ?>
				</div>
			</div>

		</div>
		<div>
			<p><?php _e( 'Debug:', 'wpify-woo' ); ?> <a href="<?php echo add_query_arg( [ 'wpify-action' => 'download-log', 'wpify-nonce' => wp_create_nonce( 'download-log' ) ], admin_url() ) ?>">Download logs</a></p>
		</div>
		<?php
	}

	/**
	 *  Get settings array
	 * @return array
	 */
	public function get_settings() {
		global $current_section;

		if ( ! $current_section ) {
			$current_section = 'general';
		}

		$settings = apply_filters( 'wpify_woo_settings_' . $current_section, array() );

		return apply_filters( 'woocommerce_get_settings_' . $this->id, $settings );
	}

	public function settings_general() {
		return array(
				array(
						'type'    => 'multiswitch',
						'id'      => 'enabled_modules',
						'label'   => __( 'Enabled modules', 'wpify-woo' ),
						'options' => $this->get_plugin()->get_woocommerce_integration()->get_modules(),
						'desc'    => __( 'Select the modules you want to enable', 'wpify-woo' ),
				),
		);
	}
}
