<?php

namespace WpifyWoo\Modules\IcDic;

use DragonBe\Vies\Vies;
use DragonBe\Vies\ViesException;
use Exception;
use h4kuna\Ares\Ares;
use h4kuna\Ares\Exceptions\IdentificationNumberNotFoundException;
use WC_Order;
use WpifyWoo\Abstracts\AbstractModule;
use WpifyWoo\Modules\IcDic\Api\IcDicApi;

/**
 * Class IcDicModule
 * @package WpifyWoo\Modules\IcDic
 */
class IcDicModule extends AbstractModule {

	/**
	 * Setup
	 * @return void
	 */
	public function setup() {
		add_filter( 'wpify_woo_settings_' . $this->id(), array( $this, 'settings' ) );
		add_filter( 'woocommerce_checkout_fields', array( $this, 'adjust_checkout_fields' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
		add_filter( 'woocommerce_default_address_fields', array( $this, 'adjust_fields_priority' ) );
		add_filter( 'woocommerce_order_formatted_billing_address', array( $this, 'add_fields_to_address' ), 10, 2 );
		add_filter( 'woocommerce_formatted_address_replacements', array( $this, 'replace_tags_in_emails' ), 10, 2 );
		add_filter( 'woocommerce_localisation_address_formats', array( $this, 'localisation_address_formats' ) );
		add_action( 'woocommerce_after_checkout_validation', array( $this, 'checkout_validation' ), 10, 2 );
		add_action( 'init', array( $this, 'add_rest_api' ) );
		add_action( 'woocommerce_checkout_before_customer_details', array( $this, 'render_ares' ) );
		add_filter( 'woocommerce_billing_fields', array( $this, 'in_vat_woocommerce_billing' ) );
		add_filter(
				'woocommerce_my_account_my_address_formatted_address',
				array(
						$this,
						'add_in_vat_to_address',
				),
				10,
				3
		);
		add_action( 'init', array( $this, 'set_customer_vat_extempt' ) );
		add_action( 'woocommerce_checkout_update_order_review', array( $this, 'set_vat_extempt_on_order_review' ) );
	}

	/**
	 * Module ID
	 * @return string
	 */
	public function id(): string {
		return 'ic_dic';
	}

	public function name() {
		return __( 'Checkout IČ and DIČ', 'wpify-woo' );
	}

	/**
	 * Enqueue frontend scripts
	 */
	public function enqueue_scripts() {
		if ( ! is_checkout() ) {
			return;
		}

		$assets = $this->plugin->get_webpack_manifest()->get_assets(
				'icdic.js',
				'wpify-woo-ic-dic',
				array(
						'wpifyWooIcDic' => array(
								'restUrl' => $this->plugin->get_api_manager()->get_rest_url(),
						),
				)
		);

		foreach ( $assets as $key => $asset ) {
			$assets[ $key ]['in_footer'] = true;
		}

		$this->plugin->get_assets()->enqueue_assets( $assets );

		$css = $this->plugin->get_webpack_manifest()->get_assets( 'icdic.css' );

		foreach ( $css as $script ) {
			wp_enqueue_style( $script['handle'], $script['file'], $script['deps'], $script['version'] );
		}
	}


	/**
	 * Add settings
	 * @return array
	 */
	public function settings(): array {
		return array(
				array(
						'id'    => 'move_company_field',
						'type'  => 'switch',
						'label' => __( 'Move company field', 'wpify-woo' ),
						'desc'  => __( 'Check if you want to move the company field to the end of the checkout form along with the extra fields', 'wpify-woo' ),
				),
				array(
						'id'    => 'show_checkbox',
						'type'  => 'switch',
						'label' => __( 'Show "Enter company details" checkbox', 'wpify-woo' ),
						'desc'  => __( 'Check if want to show the checkbox "Enter company details" - the extra fields will show only if the checkbox is checked.', 'wpify-woo' ),
				),
				array(
						'id'      => 'validate_ares',
						'type'    => 'multiswitch',
						'label'   => __( 'Validate entered identification number from ARES', 'wpify-woo' ),
						'desc'    => __( 'Check if want to validate the entered identification number with ARES.', 'wpify-woo' ),
						'options' => array(
								array(
										'label' => __( 'After entering identification number on the checkout form', 'wpify-woo' ),
										'value' => 'ic_entered',
								),
								array(
										'label' => __( 'After submitting the order', 'wpify-woo' ),
										'value' => 'order_submit',
								),
						),
				),
				array(
						'id'    => 'autofill_ares',
						'type'  => 'switch',
						'label' => __( 'Autofill from ARES', 'wpify-woo' ),
						'desc'  => __( 'Enable if you want to display "Fill automatically from ARES" at the top of checkout form.', 'wpify-woo' ),
				),
				array(
						'id'            => 'autofill_ares_text',
						'type'          => 'text',
						'label'         => __( 'Autofill from ARES text', 'wpify-woo' ),
						'desc'          => __( 'Enter the text that will appear on top of the checkout.', 'wpify-woo' ),
						'default_value' => __( 'Autofill from Ares', 'wpify-woo' ),
				),
				array(
						'id'    => 'validate_vies',
						'type'  => 'switch',
						'label' => __( 'Validate VAT number in VIES', 'wpify-woo' ),
						'desc'  => __( 'Check if want to validate the entered VAT with VIES.', 'wpify-woo' ),
				),
				array(
						'id'      => 'zero_tax_for_vat_countries',
						'type'    => 'multiselect',
						'label'   => __( 'Zero tax for VAT numbers in', 'wpify-woo' ),
						'desc'    => __( 'Select countries where you want to apply zero VAT.', 'wpify-woo' ),
						'options' => $this->get_eu_countries(),
						'multi'   => true,
				),
		);
	}

	private function get_eu_countries() {
		$countries = Vies::listEuropeanCountries();

		array_walk(
				$countries,
				function ( &$country, $code ) {
					$country = array(
							'label' => $country . ' (' . $code . ')',
							'value' => $code,
					);
				}
		);

		return array_values( $countries );
	}

	/**
	 * Adjust the checkout fields
	 *
	 * @param array $fields Array of the checkout fields.
	 *
	 * @return array
	 */
	public function adjust_checkout_fields( array $fields ): array {
		$temp_ic  = $fields['billing']['billing_ic'];
		$temp_dic = $fields['billing']['billing_dic'];

		unset( $fields['billing']['billing_ic'] );
		unset( $fields['billing']['billing_dic'] );

		$extra_billing_fields = array();
		$classes              = array( 'form-row-wide' );

		if ( $this->get_settings()['show_checkbox'] ) {
			$extra_billing_fields['company_details'] = array(
					'type'     => 'checkbox',
					'label'    => __( 'Enter company details', 'wpify-woo' ),
					'required' => false,
					'class'    => array( 'wpify-woo-ic-dic__toggle' ),
					'priority' => 200,
			);
			$classes[]                               = 'wpify-woo-ic-dic__company_field';
		}

		if ( ! empty( $this->get_setting( 'validate_ares' ) ) && in_array( 'ic_entered', $this->get_setting( 'validate_ares' ) ) ) {
			$classes[] = 'wpify-woo-ic--validate';
		}

		if ( $this->get_setting( 'move_company_field' ) ) {
			$temp_company = $fields['billing']['billing_company'];
			unset( $fields['billing']['billing_company'] );
			$extra_billing_fields['billing_company'] = array_merge(
					$temp_company,
					array(
							'class'    => $classes,
							'priority' => 210,
					)
			);
		} else {
			$fields['billing']['billing_company']['classes'] = $classes;
		}

		$extra_billing_fields['billing_ic'] = array_merge(
				$temp_ic,
				array(
						'class'    => $classes,
						'priority' => 220,
				)
		);

		$extra_billing_fields['billing_dic'] = array_merge(
				$temp_dic,
				array(
						'class'    => $classes,
						'priority' => 230,
				)
		);

		$fields['billing'] = array_merge( $fields['billing'], $extra_billing_fields );

		return $fields;
	}

	/**
	 * Adjust checkout fields priorities
	 *
	 * @param array $fields Array of the fields.
	 *
	 * @return mixed
	 */
	public function adjust_fields_priority( array $fields ): array {
		if ( $this->get_setting( 'move_company_field' ) ) {
			$fields['company']['priority'] = 210;
		}

		return $fields;
	}

	/**
	 * Add details to localisation address formats
	 *
	 * @param array $address_formats Address formats.
	 *
	 * @return mixed
	 */
	public function localisation_address_formats( array $address_formats ): array {
		foreach ( $address_formats as $key => $format ) {
			$address_formats[ $key ] = $format . "\n{billing_ic}\n{billing_dic}";
		}

		return $address_formats;
	}

	/**
	 * Add the fields values to the address
	 *
	 * @param array    $address Address.
	 * @param WC_Order $order Order.
	 *
	 * @return array
	 */
	public function add_fields_to_address( array $address, WC_Order $order ): array {
		$address['billing_ic']  = $order->get_meta( '_billing_ic', true );
		$address['billing_dic'] = $order->get_meta( '_billing_dic', true );

		return $address;
	}

	/**
	 * Replace the tags in emails
	 *
	 * @param array $replacements Array of replacements.
	 * @param array $args Array of the available args.
	 *
	 * @return array
	 */
	public function replace_tags_in_emails( array $replacements, array $args ): array {
		if ( ! empty( $args['billing_ic'] ) ) {
			$replacements['{billing_ic}'] = sprintf( '%s: %s', __( 'Identification no.', 'wpify-woo' ), $args['billing_ic'] );
		} else {
			$replacements['{billing_ic}'] = '';
		}
		if ( ! empty( $args['billing_dic'] ) ) {
			$replacements['{billing_dic}'] = sprintf( '%s: %s', __( 'VAT no.', 'wpify-woo' ), $args['billing_dic'] );
		} else {
			$replacements['{billing_dic}'] = '';
		}

		return $replacements;
	}

	/**
	 * Validate the checkout
	 *
	 * @param array $fields Array of the fields.
	 * @param       $errors
	 */
	public function checkout_validation( $fields, $errors ) {
		if ( ! $this->get_setting( 'validate_ares' ) ) {
			return;
		}
		if ( in_array( 'order_submit', $this->get_setting( 'validate_ares' ) ) && ! empty( $_POST['billing_ic'] ) ) {
			$ares = new Ares();
			$ic   = sanitize_text_field( $_POST['billing_ic'] );

			if ( ! is_numeric( $ic ) ) {
				$errors->add( 'validation', __( 'Please enter valid IC', 'wpify-woo' ) );
			} else {
				try {
					$ares->loadData( $ic );
				} catch ( IdentificationNumberNotFoundException $e ) {
					$errors->add( 'validation', __( 'The entered Company Number has not been found in ARES, please enter valid company number.', 'wpify-woo' ) );
				}
			}
		}

		if ( $this->get_settings( 'validate_vies' ) && ! empty( $_POST['billing_dic'] ) ) {
			if ( ! $this->is_valid_dic( $_POST['billing_dic'] ) ) {
				$errors->add( 'validation', __( 'The entered VAT Number has not been found in VIES, please enter valid VAT number.', 'wpify-woo' ) );
			}
		}
	}

	public function is_valid_dic( $dic ) {
		$current_country = substr( $dic, 0, 2 );
		$current_vat_no  = substr( $dic, 2 );
		$vies            = new Vies();

		try {
			if ( $vies->getHeartBeat() ) {
				return $vies->validateVat( $current_country, $current_vat_no )->isValid();
			} else {
				return $vies->validateVatSum( $current_country, $current_vat_no );
			}
		} catch ( Exception $e ) {
			return false;
		}
	}

	public function add_rest_api() {
		$api = $this->get_plugin()->create_component( IcDicApi::class );
		$api->init();
		$this->get_plugin()->get_api_manager()->add_module( $api );
	}

	public function render_ares() {
		if ( ! $this->get_setting( 'autofill_ares' ) ) {
			return;
		} ?>
		<div>
			<a href="#"
			   id="wpify-woo-icdic__ares-autofill-button"><?php echo $this->get_setting( 'autofill_ares_text' ); ?></a>
			<div class="wpify-woo-icdic__ares-autofill">
				<input type="text" name="ares_vat_no" id="ares_in"
					   placeholder="<?php _e( 'Identification number', 'wpify-woo' ); ?>"/>
				<input type="button" value="<?php _e( 'Search in ARES', 'wpify-woo' ); ?>"
					   id="wpify-woo-icdic__ares-submit"/>
				<div id="wpify-woo-icdic__ares-result"></div>
			</div>
		</div>
		<?php
	}

	public function in_vat_woocommerce_billing( $fields ) {
		$fields['billing_ic'] = array(
				'label'       => __( 'Identification no.', 'wpify-woo' ),
				'placeholder' => __( 'Your company\'s identification number', 'wpify-woo' ),
				'required'    => false,
				'type'        => 'text',
		);

		$fields['billing_dic'] = array(
				'label'       => __( 'VAT no.', 'wpify-woo' ),
				'placeholder' => __( 'Your company\'s VAT number', 'wpify-woo' ),
				'required'    => false,
				'type'        => 'text',
		);

		return $fields;
	}

	public function add_in_vat_to_address( $address, $customer_id, $name ) {
		$address[ $name . '_ic' ]  = get_user_meta( $customer_id, $name . '_ic', true );
		$address[ $name . '_dic' ] = get_user_meta( $customer_id, $name . '_dic', true );

		return $address;
	}

	public function set_customer_vat_extempt() {
		if ( is_ajax() ) {
			return;
		}

		$settings              = $this->get_settings();
		$vat_extempt_countries = $settings['zero_tax_for_vat_countries'];

		if ( empty( $vat_extempt_countries ) ) {
			return;
		}

		$dic            = empty( WC()->customer ) ? null : WC()->customer->get_meta( 'billing_dic' );
		$is_vat_extempt = null;
		$session_key    = 'is_vat_extempt:' . $dic;

		if ( ! empty( WC()->session ) ) {
			$is_vat_extempt = WC()->session->get( $session_key );
		}

		if ( $is_vat_extempt === null ) {
			$is_vat_extempt = $this->is_vat_extempt( $dic );
		}

		if ( ! empty( WC()->session ) ) {
			WC()->session->set( $session_key, $is_vat_extempt );
		}

		if ( ! empty( WC()->customer ) ) {
			WC()->customer->set_is_vat_exempt( $is_vat_extempt );
		}
	}

	public function is_vat_extempt( $dic ) {
		$settings              = $this->get_settings();
		$vat_extempt_countries = $settings['zero_tax_for_vat_countries'];
		$current_country       = substr( $dic, 0, 2 );
		$current_vat_no        = substr( $dic, 2 );
		$vies                  = new Vies();

		try {
			if ( $settings['validate_vies'] && $vies->getHeartBeat() ) {
				$is_valid = $vies->validateVat( $current_country, $current_vat_no )->isValid();
			} else {
				$is_valid = $vies->validateVatSum( $current_country, $current_vat_no );
			}
		} catch ( ViesException $e ) {
			$is_valid = false;
		}

		return $is_valid && in_array( $current_country, $vat_extempt_countries );
	}

	public function set_vat_extempt_on_order_review( $strdata ) {
		$data                  = array();
		$settings              = $this->get_settings();
		$vat_extempt_countries = $settings['zero_tax_for_vat_countries'];

		if ( empty( $vat_extempt_countries ) ) {
			return;
		}

		wp_parse_str( $strdata, $data );

		if (
				(
						isset( $data['company_details'] )
						&& $data['company_details'] === '1'
						&& ! empty( $data['billing_dic'] )
				) || (
						! isset( $data['company_details'] )
						&& ! empty( $data['billing_dic'] )
				)
		) {
			WC()->customer->set_is_vat_exempt( $this->is_vat_extempt( $data['billing_dic'] ) );
		} else {
			WC()->customer->set_is_vat_exempt( false );
		}
	}
}
