<?php

namespace WpifyWoo\Modules\PacketaShipping;

use Wpify\Core_4_0\Abstracts\AbstractComponent;
use WpifyWoo\Models\PacketaOrderModel;
use WpifyWoo\PacketeraSDK\ClassMap;
use WpifyWoo\PacketeraSDK\ServiceType\Create;
use WpifyWoo\PacketeraSDK\ServiceType\Packets;
use WpifyWoo\PacketeraSDK\StructType\PacketAttributes;
use WpifyWoo\PacketeraSDK\StructType\PacketIds;
use WpifyWoo\Plugin;
use WpifyWoo\Repositories\PacketaOrderRepository;
use WsdlToPhp\PackageBase\AbstractSoapClientBase;

/**
 * Class PacketaApi
 * @package WpifyWoo\Modules\PacketaShipping
 * @property Plugin $plugin
 */
class PacketaApi extends AbstractComponent {

	private $initialized = false;
	private $options;
	/** @var PacketaShippingModule $module */
	private $module;
	private $plugin;


	public function initialize() {
		if ( $this->initialized ) {
			return;
		}

		$this->options = array(
			AbstractSoapClientBase::WSDL_URL      => 'https://www.zasilkovna.cz/api/soap-php-bugfix.wsdl',
			AbstractSoapClientBase::WSDL_CLASSMAP => ClassMap::get(),
		);

		$this->plugin = wpify_woo();

		$this->module      = $this->plugin->get_module( PacketaShippingModule::class );
		$this->initialized = true;
	}

	/**
	 * @param PacketaOrderModel $order
	 */
	public function create_packet( PacketaOrderModel $order ) {
		$this->initialize();
		$create     = new Create( $this->options );
		$attributes = new PacketAttributes( null );
		$wc_order   = $order->get_wc_order();

		$data   = array(
			'first_name'   => $wc_order->get_shipping_first_name() ?: $wc_order->get_billing_first_name(),
			'last_name'    => $wc_order->get_shipping_last_name() ?: $wc_order->get_billing_last_name(),
			'order_number' => $wc_order->get_order_number(),
			'email'        => $wc_order->get_billing_email(),
			'phone'        => $wc_order->get_billing_phone(),
			'address_id'   => $order->get_packeta_id(),
			'total'        => $wc_order->get_total(),
			'currency'     => $wc_order->get_currency(),
		);
		$is_cod = ! empty( $this->module->get_setting( 'cod_gateways' ) ) && in_array( $wc_order->get_payment_method(), $this->module->get_setting( 'cod_gateways' ) );
		if ( $is_cod ) {
			$data['cod'] = $wc_order->get_total();
		}

		$weight = $order->get_weight();
		if ( ! $weight ) {
			$weight = $this->module->get_setting( 'order_weight' );
		}
		if ( $weight ) {
			$data['weight'] = $weight;
		}

		$data = apply_filters( 'wpify_woo_packeta_api_data', $data, $wc_order, $order );

		try {
			$attributes
				->setEshop( $this->module->get_setting( 'sender_name' ) )
				->setName( $data['first_name'] )
				->setSurname( $data['last_name'] )
				->setNumber( $data['order_number'] )
				->setEmail( $data['email'] )
				->setPhone( $data['phone'] )
				->setAddressId( $data['address_id'] )
				->setValue( $data['total'] )
				->setCurrency( $data['currency'] );
			if ( $is_cod && $data['cod'] ) {
				$attributes->setCod( $data['cod'] );
			}

			if ( $weight ) {
				$attributes->setWeight( floatval( $data['weight'] ) );
			}


			$result = $create->createPacket( $this->module->get_setting( 'api_password' ), $attributes );

			if ( $result !== false ) {
				$result = $create->getResult();
				$order->set_package_id( $result->getId() );
				$order->set_barcode( $result->getBarcode() );
				$this->plugin->get_repository( PacketaOrderRepository::class )->save( $order );
				$data = array(
					'order_id'   => $order->get_id(),
					'package_id' => $order->get_package_id(),
					'barcode'    => $order->get_barcode(),
				);
				$this->plugin->get_logger()->info(
					'Packeta - package created',
					array(
						'data' => $data,
					)
				);

				return $data;
			} else {
				foreach ( $create->getLastError() as $error ) {
					$message = sprintf( 'Packeta error: %s, Response: %s', $error->getMessage(), (string) $create->getLastResponse() );
					$this->plugin->get_logger()->info(
						$message,
						array(
							'data' => $data,
						)
					);
					$wc_order->add_order_note( $message );

					return new \WP_Error( 'error', $error->getMessage() );
				}
			}
		} catch ( \Exception $e ) {
			$message = sprintf( 'Packeta error: %s', $e->getMessage() );
			$this->plugin->get_logger()->info(
				$message,
				array(
					'data' => $data,
				)
			);
			$wc_order->add_order_note( $message );
		}
	}

	public function get_packets_labels( $ids ) {
		$ids = is_array( $ids ) ? $ids : (array) $ids;
		$this->initialize();
		$packets = new Packets( $this->options );
		if ( $packets->packetsLabelsPdf( $this->module->get_setting( 'api_password' ), new PacketIds( $ids ), 'A7 on A4', 0 ) !== false ) {
			return $packets->getResult();
		} else {
			$data = array(
				'package_ids' => $ids,
			);
			foreach ( $packets->getLastError() as $error ) {
				$message = sprintf( 'Packeta error: $s', $error->getMessage() );
				$this->plugin->get_logger()->info(
					$message,
					array(
						'data' => $data,
					)
				);

				return new \WP_Error( 'error', $error->getMessage() );
			}
		}
	}
}
