<?php

namespace WpifyWoo\Modules\PacketaShipping;

use WpifyWoo\Abstracts\AbstractModule;
use WpifyWoo\Models\PacketaOrderModel;
use WpifyWoo\Repositories\PacketaOrderRepository;
use WpifyWoo\WooCommerceIntegration;

class PacketaShippingModule extends AbstractModule {

	/**
	 * Packeta API
	 * @var PacketaApi
	 */
	private $packeta_api;

	/**
	 * PacketaShippingModule constructor.
	 *
	 * @param PacketaApi $packeta_api PacketaApi.
	 */
	public function __construct( PacketaApi $packeta_api ) {
		$this->packeta_api = $packeta_api;
		parent::__construct();
	}

	/**
	 * Setup
	 * @return void
	 */
	public function setup() {
		require __DIR__ . '/PacketaShippingMethod.php';

		add_filter( 'woocommerce_shipping_methods', array( $this, 'register_packeta_shipping_method' ) );
		add_action( 'woocommerce_after_checkout_validation', array( $this, 'checkout_validation' ), 10, 2 );
		add_action( 'woocommerce_after_shipping_rate', array( $this, 'add_branch_id_input' ), 10, 2 );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_checkout_script' ), 10, 2 );
		add_filter( 'wpify_woo_settings_' . $this->id(), array( $this, 'settings' ) );
		add_action(
				'woocommerce_checkout_update_order_meta',
				array(
						$this,
						'store_pickup_field_update_order_meta',
				),
				15,
				2
		);
		add_action( 'woocommerce_admin_order_data_after_shipping_address', array( $this, 'show_packeta_details' ) );
		add_action( 'woocommerce_email_customer_details', array( $this, 'add_packeta_to_email_notifications' ), 25, 4 );
		add_action( 'woocommerce_checkout_order_processed', array( $this, 'maybe_replace_shipping_address' ), 25, 4 );
		add_action( 'add_meta_boxes', array( $this, 'add_metaboxes' ) );
		add_action( 'admin_init', array( $this, 'maybe_send_to_packeta' ) );
		add_filter( 'bulk_actions-edit-shop_order', [ $this, 'custom_bulk_actions' ], 20, 1 );
		add_filter( 'handle_bulk_actions-edit-shop_order', [ $this, 'handle_custom_bulk_actions' ], 10, 3 );
	}

	/**
	 * Set the module ID.
	 * @return string
	 */
	public function id(): string {
		return 'packeta_shipping';
	}

	/**
	 * Set the module ID.
	 * @return string
	 */
	public function name() {
		return __( 'Packeta Shipping', 'wpify-woo' );
	}


	/**
	 * Add settings
	 * @return array[] Settings.
	 */
	public function settings(): array {
		$settings = array(
				array(
						'id'    => 'api_key',
						'type'  => 'text',
						'label' => __( 'API Key', 'wpify-woo' ),
						'desc'  => __( 'Enter your API key', 'wpify-woo' ),
				),
				array(
						'id'    => 'api_password',
						'type'  => 'text',
						'label' => __( 'API Password', 'wpify-woo' ),
						'desc'  => __( 'Enter your API password', 'wpify-woo' ),
				),
				array(
						'id'    => 'sender_name',
						'type'  => 'text',
						'label' => __( 'Sender name', 'wpify-woo' ),
						'desc'  => __( 'Enter your sender name', 'wpify-woo' ),
				),
				array(
						'id'    => 'order_weight',
						'type'  => 'text',
						'label' => __( 'Order weight', 'wpify-woo' ),
						'desc'  => __( 'Enter weight in kg that will be sent to Zasilkovna if the products in order do not have weight specified.', 'wpify-woo' ),
				),
				array(
						'id'    => 'replace_shipping_address',
						'type'  => 'switch',
						'label' => __( 'Replace shipping address', 'wpify-woo' ),
						'desc'  => __( 'Enable to replace the WooCommerce shipping address with Packeta address when an order is submitted', 'wpify-woo' ),
				),
				array(
						'id'      => 'cod_gateways',
						'type'    => 'multiselect',
						'label'   => __( 'Select the COD gateways', 'wpify-woo' ),
						'desc'    => __( 'When sending the order to Zásilkovna, the order will be marked as COD for the selected gateways. This setting does NOT add COD fee to the checkout - to setup the payment gateways fees you can use free plugin WooCommerce Pay for Payment.',
								'wpify-woo' ),
						'options' => $this->plugin->get_woocommerce_integration()->get_gateways(),
						'multi'   => true,
				),
		);

		return $settings;
	}

	/**
	 * Enqeueue the frontend scripts
	 * @throws \Wpify\Core_4_0\Exceptions\PluginException
	 */
	public function enqueue_checkout_script() {
		if ( ! is_checkout() ) {
			return;
		}

		wp_enqueue_script( 'packeta-main', 'https://widget.packeta.com/www/js/library.js', array(), null, true );
		wp_localize_script(
				'packeta-main',
				'packeta',
				array(
						'apiKey' => $this->get_setting( 'api_key' ),
				)
		);

		$assets = $this->plugin->get_webpack_manifest()->get_assets(
				'packeta.js',
				'wpify-woo-packeta',
				array(
						'wpifyWooPacketa' => array(
								'stringNone' => __( 'No branch selected', 'wpify-woo' ),
								'language'   => explode( '_', get_locale() )[0],
						),
				),
				array( 'packeta-main' )
		);
		foreach ( $assets as $key => $asset ) {
			$assets[ $key ]['in_footer'] = true;
		}

		$this->plugin->get_assets()->enqueue_assets( $assets );
	}

	/**
	 * Register the shipping method
	 *
	 * @param array $methods Shipping methods.
	 *
	 * @return array
	 */
	public function register_packeta_shipping_method( array $methods ): array {
		$methods['packeta'] = PacketaShippingMethod::class;

		return $methods;
	}

	/**
	 * Validate the checkout
	 *
	 * @param array $fields Array of the fields.
	 * @param       $errors
	 */
	public function checkout_validation( $fields, $errors ) {
		$packages = WC()->shipping()->get_packages();

		foreach ( $fields['shipping_method'] as $method ) {
			foreach ( $packages as $i => $package ) {
				if ( isset( $package['rates'][ $method ] ) ) {
					if ( strpos( $package['rates'][ $method ]->id, 'packeta' ) !== false && empty( $_POST['packeta_id'] ) ) {
						$errors->add( 'validation', __( 'Select Zásilkovna pickup point', 'wpify-woo' ) );
						continue( 2 );
					}
				}
			}
		}
	}

	/**
	 * Save branch ID
	 *
	 * @param int $order_id Order ID.
	 *
	 * @since 1.0.0
	 */
	public function store_pickup_field_update_order_meta( int $order_id ) {
		/** @var  PacketaOrderModel $order */
		$repository = $this->plugin->get_repository( PacketaOrderRepository::class );
		$order      = $repository->get( $order_id );
		if ( $order->is_packeta_shipping() ) {
			$order->set_packeta_id( sanitize_text_field( $_POST['packeta_id'] ) );
			$order->set_packeta_name( sanitize_text_field( $_POST['packeta_name'] ) );
			$order->set_packeta_street( sanitize_text_field( $_POST['packeta_street'] ) );
			$order->set_packeta_city( sanitize_text_field( $_POST['packeta_city'] ) );
			$order->set_packeta_postcode( sanitize_text_field( $_POST['packeta_postcode'] ) );
			$order->set_packeta_url( sanitize_text_field( $_POST['packeta_url'] ) );
			$repository->save( $order );
		}
	}

	/**
	 * Add the button and inputs
	 *
	 * @param $method
	 */
	public function add_branch_id_input( $method ) {
		if ( ! is_checkout() ) {
			return;
		}

		$chosen_shipping_rates = array_filter(
				WC()->session->get( 'chosen_shipping_methods' ),
				function ( $item ) {
					return strpos( $item, 'packeta' ) !== false;
				}
		);

		if ( empty( $chosen_shipping_rates ) ) {
			return;
		}

		if ( strpos( $method->id, 'packeta' ) !== false ) { ?>
			<style type="text/css">
				.wpify-woo-packeta__shipping-method #packeta-point-info {
					display: none;
				}

				.wpify-woo-packeta__shipping-method.wpify-woo-packeta__shipping-method--selected #packeta-point-info {
					display: block;
				}

				.wpify-woo-packeta__shipping-method.wpify-woo-packeta__shipping-method--selected .wpify-woo-packeta__button {
					display: none;
				}

			</style>
			<div class="wpify-woo-packeta__shipping-method">
				<button type="button" class="zasilkovna-open-widget wpify-woo-packeta__button"
						value=""><?php _e( 'Select pickup point', 'wpify-woo' ); ?></button>
				<input type="hidden" name="packeta_id" id="packeta-point-id">
				<input type="hidden" name="packeta_name" id="packeta-point-name">
				<input type="hidden" name="packeta_city" id="packeta-point-city">
				<input type="hidden" name="packeta_street" id="packeta-point-street">
				<input type="hidden" name="packeta_postcode" id="packeta-point-postcode">
				<input type="hidden" name="packeta_url" id="packeta-point-url">
				<div id="packeta-point-info">
					<div class="packeta-point-info__details"></div>
					<div class="packeta-point-info__change"><a href="#"
															   class="zasilkovna-open-widget"><?php _e( 'Change pickup point', 'wpify-woo' ); ?></a>
					</div>
				</div>
			</div>

			<?php
		}
	}

	/**
	 * Add Packeta details to the email notifications
	 *
	 * @param $order
	 * @param $sent_to_admin
	 * @param $plain_text
	 * @param $email
	 */
	public function add_packeta_to_email_notifications( $order, $sent_to_admin, $plain_text, $email ) {
		$this->show_packeta_details( $order );
	}

	/**
	 * Render the packeta details.
	 *
	 * @param \WC_Order $order WC Order.
	 */
	public function show_packeta_details( $order ) {
		/** @var PacketaOrderModel $order */
		$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( $order );
		if ( $order->is_packeta_shipping() ) {
			?>
			<h3>Zásilkovna</h3>
			<?php echo $order->get_packeta_name(); ?><br/>
			<?php echo $order->get_packeta_street(); ?><br/>
			<?php echo $order->get_packeta_city(); ?><br/>
			<?php echo $order->get_packeta_postcode(); ?><br/>
			<?php
			if ( $order->get_packeta_url() ) {
				printf( '<a href="%s">%s</a><br/>', $order->get_packeta_url(), __( 'Show pickup point details', 'wpify-woo' ) );
			}
			?>
			<br/>
			<?php
		}
	}

	/**
	 * Replace the shipping address if set in plugin settings
	 *
	 * @param int $order_id Order ID.
	 *
	 * @throws \WC_Data_Exception
	 */
	public function maybe_replace_shipping_address( int $order_id ) {
		if ( ! $this->get_setting( 'replace_shipping_address' ) ) {
			return;
		}
		/** @var PacketaOrderModel $order */
		$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( $order_id );
		if ( ! $order->is_packeta_shipping() ) {
			return;
		}

		$order->get_wc_order()->set_shipping_company( $order->get_packeta_name() );
		$order->get_wc_order()->set_shipping_address_1( $order->get_packeta_street() );
		$order->get_wc_order()->set_shipping_city( $order->get_packeta_city() );
		$order->get_wc_order()->set_shipping_postcode( $order->get_packeta_postcode() );
		$order->get_wc_order()->save();
	}

	public function add_metaboxes() {
		if ( empty( $_GET['post'] ) ) {
			return;
		}
		$post_id = (int) $_GET['post'];
		if ( ! $post_id || 'shop_order' !== get_post_type( $post_id ) ) {
			return;
		}

		$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( $post_id );
		if ( ! $order->is_packeta_shipping() ) {
			return;
		}

		add_meta_box(
				'wpify-woo-packeta',
				__( 'Packeta shipping', 'wpify-woo' ),
				array(
						$this,
						'render_packeta_metabox',
				),
				'shop_order',
				'side'
		);
	}

	/**
	 * Render the Packeta metabox
	 */
	public function render_packeta_metabox() {
		/** @var PacketaOrderModel $order */
		$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( sanitize_text_field( $_GET['post'] ) );
		if ( $order->get_package_id() ) {
			?>
			<div><strong>Packeta package ID: </strong> <?php echo $order->get_package_id(); ?></div>
			<a class="button" target="_blank"
			   href="<?php echo add_query_arg( array( 'packeta-generate-label' => $order->get_package_id() ), get_edit_post_link( sanitize_text_field( $_GET['post'] ) ) ); ?>"><?php _e( 'Download label', 'wpify-woo' ); ?></a>
		<?php } ?>
		<a class="button"
		   href="<?php echo add_query_arg( array( 'send-to-packeta' => 1 ), get_edit_post_link( sanitize_text_field( $_GET['post'] ) ) ); ?>"><?php $order->get_package_id() ? _e( 'Resend to Packeta', 'wpify-woo' ) : _e( 'Send to Packeta',
					'wpify-woo' ); ?></a>
		<?php
	}

	/**
	 * Maybe send order to packet or generate labels
	 */
	public function maybe_send_to_packeta() {
		if ( ! empty( $_GET['send-to-packeta'] ) && ! empty( $_GET['post'] ) ) {
			$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( sanitize_text_field( $_GET['post'] ) );
			$this->packeta_api->create_packet( $order );
		}
		if ( ! empty( $_GET['packeta-generate-label'] ) ) {
			$date = date( 'Y-m-dH:i:s' );
			header( 'Content-type:application/pdf' );
			header( "Content-Disposition:attachment;filename=labels-{$date}.pdf" );
			echo $this->packeta_api->get_packets_labels( $_GET['packeta-generate-label'] );
			exit();
		}
	}

	/**
	 * @return PacketaApi
	 */
	public function get_packeta_api(): PacketaApi {
		return $this->packeta_api;
	}

	public function custom_bulk_actions( $actions ) {
		$actions['wpify_woo_packeta_generate_labels'] = __( 'Packeta - Generate labels', 'wpify-woo' );

		return $actions;
	}

	/**
	 * Handle custom bulk action
	 *
	 * @param $redirect_to
	 * @param $action
	 * @param $post_ids
	 *
	 * @return mixed|string
	 */
	public function handle_custom_bulk_actions( $redirect_to, $action, $post_ids ) {
		if ( 'wpify_woo_packeta_generate_labels' === $action ) {
			$package_ids = [];
			foreach ( $post_ids as $post_id ) {
				/** @var PacketaOrderModel $order */
				$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( $post_id );
				if ( $order->get_package_id() ) {
					$package_ids[] = $order->get_package_id();
				} else {
					$result = $this->packeta_api->create_packet( $order );
					if ( ! is_wp_error( $result ) ) {
						$package_ids[] = $result['package_id'];
					}
				}
			}

			$redirect_to = add_query_arg( [ 'packeta-generate-label' => $package_ids ], admin_url() );
		}

		return $redirect_to;
	}
}
