<?php

namespace WpifyWoo\Modules\Vocative;

use WP_Error;
use Wpify\Core_4_0\Abstracts\AbstractComponent;
use WpifyWoo\Abstracts\AbstractModule;
use WpifyWoo\WooCommerceIntegration;

class VocativeModule extends AbstractModule {
	/** @var \wpdb $db */
	private $db;
	private $table;

	/**
	 * @return void
	 */
	public function setup() {
		global $wpdb;
		add_filter( 'wpify_woo_settings_' . $this->id(), array( $this, 'settings' ) );
		add_filter( 'woocommerce_mail_callback_params', array( $this, 'change_name_to_vocative' ), 20, 2 );
		$this->db    = $wpdb;
		$this->table = $this->db->prefix . 'wpify_vocative';
	}

	/**
	 * Module ID
	 *
	 * @return string
	 */
	public function id(): string {
		return 'vocative';
	}

	/**
	 * Get names from file
	 *
	 * @return mixed
	 */
	public function get_names() {
		return json_decode( file_get_contents( __DIR__ . '/names.json' ) );
	}

	/**
	 * Module settings
	 *
	 * @return array[]
	 */
	public function settings(): array {
		$settings = array(
			array(
				'id'      => 'vocative_settings',
				'type'    => 'options',
				'options' => array(
					'hide_submit_button' => true,
				),
			),
			array(
				'id'    => 'vocative_title',
				'type'  => 'title',
				'label' => __( 'Vocative in Woo Emails', 'wpify-woo' ),
				'desc'  => __( 'Automatically changes the salutation in Woo emails to vocative. Uses updated DB of names used in Czech republic.', 'wpify-woo' ),
			),
		);

		return $settings;
	}

	/**
	 * Module name
	 *
	 * @return string
	 */
	public function name(): string {
		return __( 'Emails Vocative', 'wpify-woo' );
	}

	/**
	 * Change name to vocative
	 *
	 * @param $params
	 * @param $email
	 *
	 * @return mixed
	 */
	public function change_name_to_vocative( $params, $email ) {
		if ( ! is_a( $email->object, '\Automattic\WooCommerce\Admin\Overrides\Order' ) ) {
			return $params;
		}

		$first_name = $email->object->get_billing_first_name();
		$vocative   = $this->get_vocative( $first_name );
		if ( ! $vocative ) {
			return $params;
		}

		$original_text = sprintf( __( 'Hi %s,', 'woocommerce' ), $first_name );
		$text          = str_replace( $first_name, $vocative, $original_text );
		$params[2]     = str_replace( $original_text, $text, $params[2] );

		return $params;
	}

	/**
	 * Get vocative
	 *
	 * @param $name
	 *
	 * @return string|null
	 */
	public function get_vocative( $name ) {
		if ( ! $this->table_exists() ) {
			$this->create_table();
		}

		return $this->db->get_var( $this->db->prepare( "SELECT vocative from {$this->table} WHERE first_name = %s", $name ) );
	}

	/**
	 * Check if the table exists
	 *
	 * @return bool
	 */
	public function table_exists(): bool {
		$result = $this->db->get_results(
			$this->db->prepare(
				'SELECT * FROM information_schema.tables
			WHERE table_schema = %s
            AND table_name = %s
			LIMIT 1;',
				DB_NAME,
				$this->table
			)
		);

		return ! empty( $result ) ? true : false;
	}

	/**
	 * Create the table
	 */
	public function create_table() {
		$charset_collate = $this->db->get_charset_collate();

		$sql = "CREATE TABLE `{$this->table}` (
  			first_name varchar(255) NOT NULL,
  			vocative varchar(255) NOT NULL
			) $charset_collate;";
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		dbDelta( $sql );

		$query = "INSERT INTO {$this->table} (first_name,vocative) VALUES";
		$items = array();
		foreach ( $this->get_names() as $first_name => $vocative ) {
			$items[] = sprintf( '("%s", "%s")', $first_name, $vocative );
		}

		$insert = sprintf( '%s %s;', $query, implode( ',', $items ) );
		$this->db->get_results( $insert );
	}
}
