<?php

namespace WpifyWoo;

use Wpify\Core_4_0\Abstracts\AbstractComponent;
use Wpify\Core_4_0\Exceptions\PluginException;
use WpifyWoo\Admin\Settings;

/**
 * Class WooCommerceIntegration
 * @package WpifyWoo
 * @property Plugin $plugin
 */
class WooCommerceIntegration extends AbstractComponent {

	const OPTION_NAME = 'wpify-woo-settings';

	/**
	 * Setup
	 * @return bool|void
	 */
	public function setup() {
		add_filter( 'woocommerce_get_settings_pages', array( $this, 'add_setting_page' ) );
	}

	/**
	 * Add settings page
	 *
	 * @param array $settings Array of settings pages.
	 *
	 * @return array
	 * @throws PluginException Exception.
	 */
	public function add_setting_page( array $settings ): array {
		$admin_settings = $this->plugin->create_component( Settings::class );
		$admin_settings->init();
		$settings[] = $admin_settings;

		return $settings;
	}

	/**
	 * Check if a module is enabled
	 *
	 * @param string $module Module name.
	 *
	 * @return bool
	 */
	public function is_module_enabled( string $module ): bool {
		return in_array( $module, $this->get_enabled_modules(), true );
	}

	/**
	 * Get an array of enabled modules
	 * @return array
	 */
	public function get_enabled_modules(): array {
		return $this->get_settings( 'general' )['enabled_modules'] ?? array();
	}

	/**
	 * Get settings for a specific module
	 *
	 * @param string $module Module name.
	 *
	 * @return array
	 */
	public function get_settings( string $module ): array {
		return get_option( sprintf( '%s-%s', $this::OPTION_NAME, $module ), array() );
	}

	/**
	 * Get available modules
	 */
	public function get_modules(): array {
		$modules = array(
			array(
				'label' => __( 'Async emails', 'wpify-woo' ),
				'value' => 'async_emails',
			),
			array(
				'label' => __( 'Packeta shipping', 'wpify-woo' ),
				'value' => 'packeta_shipping',
			),
			array(
				'label' => __( 'Checkout IČ and DIČ', 'wpify-woo' ),
				'value' => 'ic_dic',
			),
			array(
				'label' => __( 'Heureka ověřeno zákazníky', 'wpify-woo' ),
				'value' => 'heureka_overeno_zakazniky',
			),
			array(
				'label' => __( 'Heureka měření konverzí', 'wpify-woo' ),
				'value' => 'heureka_mereni_konverzi',
			),
			array(
				'label' => __( 'XML Feed Heureka', 'wpify-woo' ),
				'value' => 'xml_feed_heureka',
			),
			array(
				'label' => __( 'Free shipping notice', 'wpify-woo' ),
				'value' => 'free_shipping_notice',
			),
			array(
				'label' => __( 'Emails Vocative', 'wpify-woo' ),
				'value' => 'vocative',
			),
		);

		$modules = apply_filters( 'wpify_woo_modules', $modules );

		foreach ( $this->plugin->get_premium()->get_extensions() as $extension ) {
			$exists = array_filter( $modules, function ( $module ) use ( $extension ) {
				return $extension['id'] === $module['value'];
			} );
			if ( empty( $exists ) ) {
				$modules[] = [
					'label'    => sprintf( '<a href="%1$s"><strong>%2$s</strong></a> - %3$s <span class="wpify-woo-settings__premium"><a href="%1$s">%4$s</a></span>', $extension['url'], $extension['title'],$extension['short_description'],  __( 'Get the addon', 'wpify-woo' ) ),
					'value'    => $extension['id'],
					'disabled' => true,
				];
			}
		}


		return $modules;
	}

	public function get_avaliable_shipping_methods() {
		$shipping_methods = array();

		$zones = \WC_Shipping_Zones::get_zones();
		//$default_zone = \WC_Shipping_Zones::get_zone_by('zone_id',0);

		foreach ( $zones as $zone ) {
			$name = $zone['zone_name'];
			foreach ( $zone['shipping_methods'] as $shipping ) {
				$shipping_methods[ $shipping->id . ':' . $shipping->instance_id ] = $name . ': ' . $shipping->method_title;
			}
		}

		return $shipping_methods;
	}

	public function get_gateways() {
		$gateways           = array();
		$available_gateways = WC()->payment_gateways()->payment_gateways();
		foreach ( $available_gateways as $key => $gateway ) {
			$gateways[] = array(
				'label' => $gateway->title,
				'value' => $key,
			);
		}

		return $gateways;
	}

	public function get_order_statuses() {
		$statuses = wc_get_order_statuses();
		$result   = [];
		foreach ( $statuses as $id => $label ) {
			$result[] = [
				'label' => $label,
				'value' => str_replace( 'wc-', '', $id ),
			];
		}

		return $result;
	}
}
